from typing import Type

from ts_t1_validator.models.enums.abstract_enum import AbstractEnum
from ts_t1_validator.validators.exceptions import ValidationException
from ts_t1_validator.validators.rules.abstract_rule import ValidationRule


class OneOfEnumRule(ValidationRule):
    def __init__(self, value, expected_enum: Type[AbstractEnum], field_name: str):
        self.value = value
        self.expected_enum = expected_enum
        self.field_name = field_name

    def execute(self):
        """
        Value must not to be undefined
        """

        if not isinstance(self.value, AbstractEnum):
            raise ValidationException(f"{self.field_name} should have valid value")

        if self.value is self.value.UNDEFINED:
            raise ValidationException(f"{self.field_name} should be one of {str(self.expected_enum.asList())}")



from enum import unique

from ts_t1_validator.models.enums.abstract_enum import AbstractEnum


@unique
class LocationTypeEnum(AbstractEnum):
    DMAX = "DMAX"
    REGION = "REGN"
    UNDEFINED = None



from typing import List

from ts_t1_validator.models.enums.location_type import LocationTypeEnum


class TargetingLocationDTO:
    def __init__(self, location_type: LocationTypeEnum, included: List[int], excluded: List[int]):
        self.location_type = location_type
        self.included = included
        self.excluded = excluded

    @staticmethod
    def fromDict(user_data: dict) -> "TargetingLocationDTO":
        """
        transform user data into inner object
        :param user_data: dict
        :return:
        """
        return TargetingLocationDTO(location_type=LocationTypeEnum.set(user_data.get("location_type")),
                                    included=user_data.get("included"),
                                    excluded=user_data.get("excluded"))



import json
import os
from typing import List

from ts_t1_validator.models.enums.location_type import LocationTypeEnum
from ts_t1_validator.validators.exceptions import ValidationException
from ts_t1_validator.validators.rules.abstract_rule import ValidationRule


class TargetingLocationsInList(ValidationRule):
    REGION_PATH = "targeting_location/fixtures/regions.json"
    DMAX_PATH = "targeting_location/fixtures/dmas.json"

    def __init__(self, values: List[int], location_type: LocationTypeEnum, field_name: str):
        self.values = values
        self.location_type = location_type
        self.field_name = field_name
        self.file_path = self.__setFilePath(self.location_type)

    def __setFilePath(self, location_type: LocationTypeEnum) -> str:
        """
        set file path based on location_type
        :param location_type:
        :return: str
        """

        dir_name = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        json_path = self.REGION_PATH if location_type is LocationTypeEnum.REGION else self.DMAX_PATH
        return f"{dir_name}/{json_path}"

    def execute(self):
        """
        based on location_type value should be in one of lists
        """
        errors = []
        with open(self.file_path) as json_file:
            id_list = json.load(json_file)
            for value in self.values:
                if value not in id_list:
                    errors.append(value)

        if errors:
            raise ValidationException(f"Some ids - {errors} does not exist for {self.field_name} section in {self.location_type.value}")



from typing import Any, List

from ts_t1_validator.validators.exceptions import ValidationException
from ts_t1_validator.validators.rules.abstract_rule import ValidationRule
from ts_t1_validator.validators.rules.positive_id_rule import PositiveIDRule


class PositiveIDListRule(ValidationRule):
    def __init__(self, values: List[Any], field_name: str):
        self.values = values
        self.field_name = field_name

    def execute(self):
        """
        Provided value must be a positive integer
        """

        errors = []
        for value in self.values:
            try:
                PositiveIDRule(value=value, field_name=self.field_name).execute()
            except ValidationException:
                errors.append(value)

        if errors:
            raise ValidationException(f"{self.field_name} must be a list of valid values, found {errors}")
